@@ -164,7 +164,7 @@ class rcube_cache
         // Remove all keys
         if ($key === null) {
             $this->cache         = array();
-            $this->cache_changed = false;
+            $this->cache_changed = true;
             $this->cache_changes = array();
             $this->cache_sums    = array();
         }
@@ -260,7 +260,15 @@ class rcube_cache
         }
 
         if ($this->type != 'db') {
-            if ($this->type == 'memcache') {
+            $this->load_index();
+
+            // Consistency check (#1490390)
+            if (!in_array($key, $this->index)) {
+                // we always check if the key exist in the index
+                // to have data in consistent state. Keeping the index consistent
+                // is needed for keys delete operation when we delete all keys or by prefix.
+            }
+            else if ($this->type == 'memcache') {
                 $data = $this->db->get($this->ckey($key));
             }
             else if ($this->type == 'apc') {
@@ -336,7 +344,15 @@ class rcube_cache
         }
 
         if ($this->type == 'memcache' || $this->type == 'apc') {
-            return $this->add_record($this->ckey($key), $data);
+            $result = $this->add_record($this->ckey($key), $data);
+
+            // make sure index will be updated
+            if ($result && !array_key_exists($key, $this->cache_sums)) {
+                $this->cache_changed    = true;
+                $this->cache_sums[$key] = true;
+            }
+
+            return $result;
         }
 
         $key_exists = array_key_exists($key, $this->cache_sums);
@@ -439,13 +455,12 @@ class rcube_cache
     /**
      * Adds entry into memcache/apc DB.
      *
-     * @param string  $key   Cache key name
-     * @param mxied   $data  Serialized cache data
-     * @param bollean $index Enables immediate index update
+     * @param string $key  Cache key name
+     * @param mixed  $data Serialized cache data
      *
      * @param boolean True on success, False on failure
      */
-    private function add_record($key, $data, $index=false)
+    private function add_record($key, $data)
     {
         if ($this->type == 'memcache') {
             $result = $this->db->replace($key, $data, MEMCACHE_COMPRESSED, $this->ttl);
@@ -458,17 +473,6 @@ class rcube_cache
             $result = apc_store($key, $data, $this->ttl);
         }
 
-        // Update index
-        if ($index && $result) {
-            $this->load_index();
-
-            if (array_search($key, $this->index) === false) {
-                $this->index[] = $key;
-                $data = serialize($this->index);
-                $this->add_record($this->ikey(), $data);
-            }
-        }
-
         return $result;
     }
 
@@ -511,10 +515,15 @@ class rcube_cache
 
         // Make sure index contains new keys
         foreach ($this->cache as $key => $value) {
-            if ($value !== null) {
-                if (array_search($key, $this->index) === false) {
-                    $this->index[] = $key;
-                }
+            if ($value !== null && !in_array($key, $this->index)) {
+                $this->index[] = $key;
+            }
+        }
+
+        // new keys added using self::write()
+        foreach ($this->cache_sums as $key => $value) {
+            if ($value === true && !in_array($key, $this->index)) {
+                $this->index[] = $key;
             }
         }
 
@@ -605,8 +614,10 @@ class rcube_cache
             $this->max_packet = 2097152; // default/max is 2 MB
 
             if ($this->type == 'db') {
-                $value = $this->db->get_variable('max_allowed_packet', 1048500);
-                $this->max_packet = min($value, $this->max_packet) - 2000;
+                if ($value = $this->db->get_variable('max_allowed_packet', $this->max_packet)) {
+                    $this->max_packet = $value;
+                }
+                $this->max_packet -= 2000;
             }
             else if ($this->type == 'memcache') {
                 $stats = $this->db->getStats();
